package codeOfValue;

import edu.rice.cs.mint.runtime.Code;
import edu.rice.cs.mint.runtime.MintSerializer;
import edu.rice.cs.mint.util.Benchmark;
import edu.rice.cs.mint.util.Benchmark.Task;

import java.io.*;

public class TimingMoreAccurate {
    public static void help() {
        System.err.println(TimingMoreAccurate.class.getName()+" <file> <int> <seconds>");
    }

    public static void main(String[] args) throws IOException {
        if (args.length<2) {
            help();
        }
        else {
            File infile = new File(args[0]);
            if (!infile.exists()) {
                System.err.println("Input file "+infile+" does not exist.");
                help();
            }
            else {
                File outfile = new File(infile.getParentFile(), infile.getName()+".jar");
                DataTypes.Value ux;
                StagedDataTypes.Value sx;
                try {
                    ux = parseUnstagedArgument(args[1]);
                    sx = parseStagedArgument(args[1]);
                    
                    float time = 0.1f;
                    if (args.length>2) {
                        try {
                            time = new Float(args[2]);
                        }
                        catch(NumberFormatException nfe) {
                            System.err.println("The 3nd argument was not a float: "+args[2]);
                            help();
                            System.err.println("Continuing with time = "+time);
                        }
                    }
                    Benchmark.TARGET_SECONDS = time;
                    
                    String source = util.IO.readFile(infile);
                    
                    final DataTypes.Value uFunArgument = ux;
                    final StagedDataTypes.Value sFunArgument = sx;
                    
                    final DataTypes.Exp uFunBody = DataTypes.PARSER.parse(source);
                    final DataTypes.ValueFun uFun = new DataTypes.ValueFun() {
                        public DataTypes.Value apply(final DataTypes.Value param) {
                            return new DataTypes.Program
                                (new DataTypes.App("f", new DataTypes.Val(param)),
                                 new DataTypes.Declaration("f", "x", uFunBody))
                                .peval(DataTypes.env0, DataTypes.fenv0);
                        }
                    };
                    
                    final StagedDataTypes.Exp sFunBody = StagedDataTypes.PARSER.parse(source);
                    Code<? extends StagedDataTypes.ValueFun> codeFun = <| new StagedDataTypes.ValueFun() {
                        public StagedDataTypes.Value apply(final StagedDataTypes.Value param) {
                            return `(StagedDataTypes.bindParameter
                                         (sFunBody, <|(StagedDataTypes.Value) new StagedDataTypes.IntValue(param.intValue())|>));
                        }
                    } |>;
                    
                    String className = "LintFun_"+java.util.UUID.randomUUID().toString().replace('-','_');
                    MintSerializer.save(codeFun, className, outfile);
                    
                    final StagedDataTypes.ValueFun sFun = MintSerializer.load(outfile);
                    
                    Task unstaged = new Task("Unstaged") {
                        public void run() {
                            uFun.apply(uFunArgument);
                        }
                    };
                    Task staged = new Task("Staged") {
                        public void run() {
                            sFun.apply(sFunArgument);
                        }
                    };
                    
                    System.out.println(source);
                    Benchmark.benchmarkAndPrint(1, unstaged, staged);                    
                }
                catch(NumberFormatException nfe) {
                    System.err.println("The 2nd argument was not an integer: "+args[1]);
                    help();
                }
            }
        }
    }
    
    public static DataTypes.Value parseUnstagedArgument(String arg) {
        try {
            Integer i = new Integer(arg);
            return new DataTypes.IntValue(i);
        }
        catch(NumberFormatException nfe) {
            return new DataTypes.BooleanValue(new Boolean(arg));
        }
    }
    public static StagedDataTypes.Value parseStagedArgument(String arg) {
        try {
            Integer i = new Integer(arg);
            return new StagedDataTypes.IntValue(i);
        }
        catch(NumberFormatException nfe) {
            return new StagedDataTypes.BooleanValue(new Boolean(arg));
        }
    }
}
