package gadts;

import edu.rice.cs.mint.runtime.Code;
import edu.rice.cs.mint.runtime.MintSerializer;
import edu.rice.cs.mint.util.Benchmark;
import edu.rice.cs.mint.util.Benchmark.Task;

import java.io.*;

import gadts.ParLoops;

public class TimingParVsSer {
    public static void help() {
        System.err.println(TimingParVsSer.class.getName()+" <serfile> <parfile> <int> <cores:int> <seconds:float>");
    }

    public static void main(String[] args) throws IOException {
        if (args.length<3) {
            help();
        }
        else {
            File serfile = new File(args[0]);
            if (!serfile.exists()) {
                System.err.println("Input file "+serfile+" does not exist.");
                help();
            }
            else {
                File seroutfile = new File(serfile.getParentFile(), serfile.getName()+".jar");
                File parfile = new File(args[1]);
                if (!parfile.exists()) {
                    System.err.println("Input file "+parfile+" does not exist.");
                    help();
                }
                else {
                    File paroutfile = new File(parfile.getParentFile(), parfile.getName()+".jar");
                    gadts.ParLoops.Value ux;
                    StagedDataTypes.Value sx;
                    try {
                        sx = parseStagedArgument(args[2]);
                        ux = parseUnstagedArgument(args[2]);
                        
                        int cores = Runtime.getRuntime().availableProcessors();    ;
                        if (args.length>3) {
                            try {
                                cores = new Integer(args[3]);
                            }
                            catch(NumberFormatException nfe) {
                                System.err.println("The 4th argument was not an int: "+args[3]);
                                help();
                                System.err.println("Continuing with cores = "+cores);
                            }
                        }
                        StagedDataTypes.NUM_THREADS = cores;
                        float time = 0.1f;
                        if (args.length>4) {
                            try {
                                time = new Float(args[4]);
                            }
                            catch(NumberFormatException nfe) {
                                System.err.println("The 5th argument was not a float: "+args[4]);
                                help();
                                System.err.println("Continuing with time = "+time);
                            }
                        }
                        Benchmark.TARGET_SECONDS = time;
                        
                        String sersource = util.IO.readFile(serfile);
                        String parsource = util.IO.readFile(parfile);
                        
                        final gadts.ParLoops.Value uFunArgument = ux;
                        final StagedDataTypes.Value sFunArgument = sx;
                        
                        final gadts.ParLoops.Exp uFunBody = gadts.ParLoops.PARSER.parse(sersource);
                        final gadts.ParLoops.ValueFun uFun = new gadts.ParLoops.ValueFun() {
                            public ParLoops.Value apply(final gadts.ParLoops.Value param) {
                                return new ParLoops.Program
                                    (new ParLoops.App("f", new ParLoops.Val(param)),
                                     new ParLoops.Declaration("f", "n", uFunBody))
                                    .peval(ParLoops.env0, ParLoops.fenv0);
                            }
                        };
                        
                        final StagedDataTypes.Value serFunArgument = sx;
                        final StagedDataTypes.Value parFunArgument = sx;
                        
                        final StagedDataTypes.Exp serFunBody = StagedDataTypes.PARSER.parse(sersource);
                        final StagedDataTypes.ProgramIR pir = StagedDataTypes.bindFunction("m", "f", "n", serFunBody);
                        Code<? extends StagedDataTypes.Fun<StagedDataTypes.Value,StagedDataTypes.Value>> serCodeFun = 
                            <| (StagedDataTypes.Fun<StagedDataTypes.Value,StagedDataTypes.Value>) 
                            new StagedDataTypes.Fun<StagedDataTypes.Value,StagedDataTypes.Value>() {
                            public StagedDataTypes.Value apply (StagedDataTypes.Value input) {
                                return `(StagedDataTypes.evalFunction("m", <| input |>, pir));
                            }
                        } |>;

                        
                        String serClassName = "SerialLintFun_"+java.util.UUID.randomUUID().toString().replace('-','_');
                        MintSerializer.save(serCodeFun, serClassName, seroutfile);
                        
                        final StagedDataTypes.Fun<StagedDataTypes.Value,StagedDataTypes.Value> serFun =
                            MintSerializer.load(seroutfile);

                        final StagedDataTypes.Exp parFunBody = StagedDataTypes.PARSER.parse(parsource);
                        final StagedDataTypes.ProgramIR ppir = StagedDataTypes.bindFunction("m", "f", "n", parFunBody);
                        Code<? extends StagedDataTypes.Fun<StagedDataTypes.Value,StagedDataTypes.Value>> parCodeFun = 
                            <| (StagedDataTypes.Fun<StagedDataTypes.Value,StagedDataTypes.Value>) 
                            new StagedDataTypes.Fun<StagedDataTypes.Value,StagedDataTypes.Value>() {
                            public StagedDataTypes.Value apply (StagedDataTypes.Value input) {
                                return `(StagedDataTypes.evalFunction("m", <| input |>, ppir));
                            }
                        } |>;

                        
                        String parClassName = "ParallelLintFun_"+java.util.UUID.randomUUID().toString().replace('-','_');
                        MintSerializer.save(parCodeFun, parClassName, paroutfile);
                        
                        final StagedDataTypes.Fun<StagedDataTypes.Value,StagedDataTypes.Value> parFun =
                            MintSerializer.load(paroutfile);

                        
                        Task unstaged = new Task("Unstaged Serial ") {
                            public void run() {
                                uFun.apply(uFunArgument);
                            }
                        };
                        Task serstaged = new Task("Staged Serial  ") {
                            public void run() {
                                serFun.apply(serFunArgument);
                            }
                        };
                        Task parstaged = new Task("Staged Parallel") {
                            public void run() {
                                parFun.apply(parFunArgument);
                            }
                        };
                        
                        System.out.println("Serial:\n"+sersource);
                        System.out.println("Parallel:\n"+parsource);
                        System.out.println("Cores: "+StagedDataTypes.NUM_THREADS+
                                           " (system has "+Runtime.getRuntime().availableProcessors()+" processors)");
                        Benchmark.benchmarkAndPrint(2, unstaged, serstaged, parstaged);                    
                    }
                    catch(NumberFormatException nfe) {
                        System.err.println("The 3nd argument was not an integer: "+args[2]);
                        help();
                    }
                }
            }
        }
    }
    
    public static ParLoops.Value parseUnstagedArgument(String arg) {
        try {
            Integer i = new Integer(arg);
            return new ParLoops.IntValue(i);
        }
        catch(NumberFormatException nfe) {
            return new ParLoops.BooleanValue(new Boolean(arg));
        }
    }
    public static StagedDataTypes.Value parseStagedArgument(String arg) {
        try {
            Integer i = new Integer(arg);
            return new StagedDataTypes.IntValue(i);
        }
        catch(NumberFormatException nfe) {
            return new StagedDataTypes.BooleanValue(new Boolean(arg));
        }
    }
}
