package simple_int;

import edu.rice.cs.mint.runtime.Code;
import edu.rice.cs.mint.runtime.MintSerializer;
import edu.rice.cs.mint.util.Benchmark;
import edu.rice.cs.mint.util.Benchmark.Task;

import java.io.*;

public class TimingMoreAccurate {
    public static void help() {
        System.err.println(TimingMoreAccurate.class.getName()+" <file> <int> <seconds>");
    }

    public static void main(String[] args) throws IOException {
        if (args.length<2) {
            help();
        }
        else {
            File infile = new File(args[0]);
            if (!infile.exists()) {
                System.err.println("Input file "+infile+" does not exist.");
                help();
            }
            else {
                File outfile = new File(infile.getParentFile(), infile.getName()+".jar");
                int x = 0;
                try {
                    x = new Integer(args[1]);
                    float time = 0.1f;
                    if (args.length>2) {
                        try {
                            time = new Float(args[2]);
                        }
                        catch(NumberFormatException nfe) {
                            System.err.println("The 3nd argument was not a float: "+args[2]);
                            help();
                            System.err.println("Continuing with time = "+time);
                        }
                    }
                    Benchmark.TARGET_SECONDS = time;
                    
                    String source = util.IO.readFile(infile);
                    
                    final int funArgument = x;
                    final UnstagedLint.Exp uFunBody = parser.LintParser.parse(UnstagedLint.Exp.class, source);
                    final UnstagedLint.IntFun uFun = new UnstagedLint.IntFun() {
                        public int apply(final int param) {
                            return new UnstagedLint.Program
                                (new UnstagedLint.App("f", new UnstagedLint.Int(param)),
                                 new UnstagedLint.Declaration("f", "x", uFunBody))
                                .peval(UnstagedLint.env0, UnstagedLint.fenv0);
                        }
                    };
                    
                    final StagedLint.Exp sFunBody = parser.LintParser.parse(StagedLint.Exp.class, source);
                    Code<? extends StagedLint.IntFun> codeFun = <| new StagedLint.IntFun() {
                        public int apply(final int param) {
                            return `(StagedLint.bindParameter(sFunBody, <|param|>));
                        }
                    } |>;
                    
                    String className = "LintFun_"+java.util.UUID.randomUUID().toString().replace('-','_');
                    MintSerializer.save(codeFun, className, outfile);
                    
                    final StagedLint.IntFun sFun = MintSerializer.load(outfile);
                    
                    Task unstaged = new Task("Unstaged") {
                        public void run() {
                            uFun.apply(funArgument);
                        }
                    };
                    Task staged = new Task("Staged") {
                        public void run() {
                            sFun.apply(funArgument);
                        }
                    };
                    
                    System.out.println(source);
                    Benchmark.benchmarkAndPrint(1, unstaged, staged);                    
                }
                catch(NumberFormatException nfe) {
                    System.err.println("The 2nd argument was not an integer: "+args[1]);
                    help();
                }
            }
        }
    }
}
