package util;

import edu.rice.cs.mint.runtime.Code;
import edu.rice.cs.mint.runtime.MintSerializer;

import java.io.*;

public abstract class ACompiler<E,F> {
    public void help() {
        System.err.println(getClass().getName()+" [-verbose] <infile> [<outfile>]");
    }

    public void run(String[] args) throws IOException {
        boolean verbose = false;
        File infile;
        File outfile;        
        if (args.length==0) { help(); return; }

        // handle options
        int argIndex = 0;
        while(argIndex<args.length) {
            String a = args[argIndex];
            if (a.equals("-verbose")) { verbose = true; }
            else { break; }
            ++argIndex;
        }
        
        // handle infile
        if (argIndex>=args.length) {
            System.err.println("Input file not specified."); help(); return;
        }

        infile = new File(args[argIndex++]);
        if (!infile.exists()) {
            System.err.println("Input file "+infile+" does not exist."); help(); return;
        }

        // handle outfile
        if (argIndex<args.length) {
            // outfile specified
            outfile = new File(args[argIndex++]);
        }
        else {
            // outfile not specified, base on infile
            outfile = new File(infile.getParentFile(), infile.getName()+".jar");
        }
        
        String source = util.IO.readFile(infile);
        if (verbose) {
            System.out.println("--- Source: "+infile);
        }
        
        final E funBody = parse(source);
        Code<F> codeFun = getFunction(funBody);
        
        String className = "LintFun_"+java.util.UUID.randomUUID().toString().replace('-','_');
        
        if (verbose) {
            System.out.println(source);
            System.out.println("--- Compiled to: "+outfile);
            MintSerializer.save(codeFun, className, outfile, MintSerializer.Options.VERBOSE);
        }
        else {
            MintSerializer.save(codeFun, className, outfile);
        }
    }

    public abstract E parse(String source);
    public abstract Code<F> getFunction(E funBody);
}
