package util;

import edu.rice.cs.mint.runtime.Code;
import edu.rice.cs.mint.runtime.MintSerializer;
import edu.rice.cs.mint.util.Benchmark;
import edu.rice.cs.mint.util.Benchmark.Task;

import java.io.*;

public abstract class ATiming<UE,UA,UR,SE,SF,SA,SR> {
    AInterpreter<UE,UA,UR> _interpreter;
    ACompiler<SE,SF> _compiler;
    ARunner<SF,SA,SR> _runner;
    
    public ATiming(AInterpreter<UE,UA,UR> interpreter,
                   ACompiler<SE,SF> compiler,
                   ARunner<SF,SA,SR> runner) {
        _interpreter = interpreter;
        _compiler = compiler;
        _runner = runner;
    }
    
    public void help() {
        System.err.println(getClass().getName()+" <file> <int> <seconds>");
    }

    public void run(String[] args) throws IOException {
        if (args.length<2) { help(); return; }

        File infile = new File(args[0]);
        if (!infile.exists()) {
            System.err.println("Input file "+infile+" does not exist.");
            help();
            return;
        }

        File outfile = new File(infile.getParentFile(), infile.getName()+".jar");
        
        UA ux;
        SA sx;
        try {
            ux = _interpreter.parseArgument(args[1]);
            sx = _runner.parseArgument(args[1]);
        }
        catch(NumberFormatException nfe) {
            System.err.println("The 2nd argument was not formatted correctly: "+args[1]);
            help();
            return;
        }
        
        float time = 0.1f;
        if (args.length>2) {
            try {
                time = new Float(args[2]);
            }
            catch(NumberFormatException nfe) {
                System.err.println("The 3nd argument was not a float: "+args[2]);
                help();
                System.err.println("Continuing with time = "+time);
            }
        }
        Benchmark.TARGET_SECONDS = time;
        
        final UA uFunArgument = ux;
        final SA sFunArgument = sx;
        
        String source = IO.readFile(infile);
        final UE uFunBody = _interpreter.parse(source);       

        final SE sFunBody = _compiler.parse(source);
        Code<SF> codeFun = _compiler.getFunction(sFunBody);
        
        String className = "LintFun_"+java.util.UUID.randomUUID().toString().replace('-','_');
        MintSerializer.save(codeFun, className, outfile);
        
        final SF sFun = MintSerializer.load(outfile);
        
        Task unstaged = new Task("Unstaged") {
            public void run() {
                _interpreter.interpret(uFunBody, uFunArgument);
            }
        };
        Task staged = new Task("Staged") {
            public void run() {
                _runner.execute(sFun, sFunArgument);
            }
        };
        
        System.out.println(source);
        Benchmark.benchmarkAndPrint(1, unstaged, staged);                    
    }
}
