package util;

import java.io.*;

public class IO {
    private static String[] _input = new String[0];
    private static int _inputIndex = 0;
    private static String[] _output = new String[0];
    
    public static synchronized separable String getInput() {
        if (_inputIndex<_input.length) {
            return _input[_inputIndex++];
        }
        else {
            throw new RuntimeException("No input available");
        }
    }
    
    public static synchronized separable void addInput(String... args) {
        int remaining = _input.length - _inputIndex;
        String[] newInput = new String[args.length + remaining];
        for(int i=0; i<remaining; ++i) { newInput[i] = _input[_inputIndex+i]; }
        for(int i=0; i<args.length; ++i) { newInput[remaining+i] = args[i]; }
        _input = newInput;
        _inputIndex = 0;
    }
    
    public static synchronized void showInput() {
        for(int i=0; i<_input.length; ++i) {
            System.out.println("Input "+i+" ready: "+_input[i]);
        }
    }

    public static synchronized separable void addOutput(String... args) {
        String[] newOutput = new String[args.length + _output.length];
        for(int i=0; i<_output.length; ++i) { newOutput[i] = _output[i]; }
        for(int i=0; i<args.length; ++i) { newOutput[_output.length] = args[i]; }
        _output = newOutput;
    }

    public static synchronized void addOutput(Throwable t) {
        StringWriter sw = new StringWriter();
        t.printStackTrace(new PrintWriter(sw));
        addOutput(sw.toString());
    }
    
    public static synchronized void flushOutput() {
        for(String s: _output) { System.out.println(s); }
        _output = new String[0];
    }

    public static String readFile(File f) throws IOException {
        StringWriter sw = new StringWriter();
        BufferedReader br = null;
        try {
            br = new BufferedReader(new FileReader(f));
            final int BUF_SIZE = 1024;
            char[] buf = new char[BUF_SIZE];
            int numRead = 0;
            while((numRead=br.read(buf)) != -1) {
                sw.write(buf, 0, numRead);
            }
            br.close();
            return sw.toString();
        }
        finally {
            if (br!=null) try { br.close(); } catch(IOException ignored) { }
        }
    }

    
    public static void main(String[] args) {
        System.out.println(getInput()); // null
        addInput("1", "2", "3");
        System.out.println(getInput()); // 1
        addInput("4", "5", "6");
        System.out.println(getInput()); // 2
        System.out.println(getInput()); // 3
        System.out.println(getInput()); // 4
        System.out.println(getInput()); // 5
        System.out.println(getInput()); // 6
        System.out.println(getInput()); // null
    }
}
