package util;

public class Misc {
    public static void displayImages(final int imageSize, String title, final double[]... images) {
        javax.swing.JFrame frame = new javax.swing.JFrame(title);
        frame.setSize(imageSize * images.length+100, imageSize+100);
        frame.setDefaultCloseOperation(javax.swing.JFrame.EXIT_ON_CLOSE);
        frame.add(new javax.swing.JPanel() {
            public void paintComponent(java.awt.Graphics g) {
                for(int i=0; i<images.length; ++i) {
                    int offset = 0;
                    for(int y=0; y<imageSize; ++y) {
                        for(int x=0; x<imageSize; ++x) {
                            float f = (float)images[i][offset++];
                            g.setColor(new java.awt.Color(f,f,f));
                            g.fillRect(50+i*imageSize+x,50+y,1,1);
                        }
                    }
                }
            }
        });
        frame.setVisible(true);        
    }
    
    /**
     * Make a Gaussian blur kernel. rows should be odd
     * Based on http://www.jhlabs.com/ip/blurring.html
     */
    public static double[] makeKernel(int rows) {
        // int r = (int)Math.ceil(radius);
        // int rows = r*2+1;
        double radius = (rows-1)/2;
        int r = (int)radius;
        double[] matrix = new double[rows];
        double sigma = radius/3;
        double sigma22 = 2*sigma*sigma;
        double sigmaPi2 = 2*Math.PI*sigma;
        double sqrtSigmaPi2 = (double)Math.sqrt(sigmaPi2);
        double radius2 = radius*radius;
        double total = 0;
        int index = 0;
        for (int row = -r; row <= r; ++row) {
            double distance = row*row;
            if (distance > radius2) {
                matrix[index] = 0;
            }
            else {
                matrix[index] = (float)Math.exp(-(distance)/sigma22) / sqrtSigmaPi2;
            }
            total += matrix[index];
            ++index;
        }
        for (int i = 0; i < rows; ++i) {
            matrix[i] /= total;
        }
        
        return matrix;
    }
    
    public static <X> X printAndReturn(X x, String s) {
        util.IO.addOutput(s);
        return x;
    }
    public static <X> X printValueAndReturn(X x, String s) {
        util.IO.addOutput(s+": "+x);
        return x;
    }
}